#include <avr/io.h>  // required for TCCR3B and other ports
#include <avr/sfr_defs.h> // required for _BV
#include "rprintf.h"
#include "uart4.h"
#include "timer640.h"
#include "a2d.h"

#define PORT_ON( port_letter, number )			port_letter |= (1<<number)
#define PORT_OFF( port_letter, number )			port_letter &= ~(1<<number)
#define PWM3_PRESCALER 64
#define HS311_FREQ_MS 20
#define SERVO_CENTER 1.5
#define SERVO_MAX_LEFT 0.75
#define SERVO_MAX_RIGHT 2.25
//374.925
#define WHEEL_STOP_POSITION (ICR3 * SERVO_CENTER / HS311_FREQ_MS)
//187.463
#define MIN_SPEED_VALUE (ICR3 * SERVO_MAX_LEFT / HS311_FREQ_MS)
//624.875
#define MAX_SPEED_VALUE (ICR3 * SERVO_MAX_RIGHT / HS311_FREQ_MS)

/*
ATmega640: Four 8-bit PWM Channels, Six/Twelve PWM
Channels with Programmable Resolution from 2 to 16 Bits


PWM pins on Axon:
OC0A  B7 (not connected)
OC0B  G5 (attached to button)
OC1A  B5 (not connected)
OC1B  B6 (attached to green LED)
OC1C  B7 (not connected)
OC2A  B4 (not connected)
OC2B  H6
OC3A  E3
OC3B  E4
OC3C  E5
OC4A  H3
OC4B  H4
OC4C  H5
OC5A  L3 (not connected)
OC5B  L4 (not connected)
OC5C  L5 (not connected)

explanations:
http://www.societyofrobots.com/robotforum/index.php?topic=1827.0
http://www.societyofrobots.com/robotforum/index.php?topic=5590.0

If you use this, then you cannot use the associated timers for other things.
H6 	 uses timer2
E3-5 uses timer3
H3-5 uses timer4
Unfortunately, the 3 pins that use timer5 are not connected on the Axon board, so they are
not available for PWM. The upside is that timer5 is available for other use without interference.
*/

uint16_t get_top_fast_pwm(const uint16_t timer_prescaler, const uint16_t frequency) {
  return (uint16_t)((F_CPU/(timer_prescaler * frequency)) - 1);
}

// speed is from -187 to +187
void set_left_wheel_speed(const int16_t speed)
{
  OCR3A = (WHEEL_STOP_POSITION + speed) > MAX_SPEED_VALUE ? MAX_SPEED_VALUE:(WHEEL_STOP_POSITION + speed);
}
// speed is from -187 to +187
void set_right_wheel_speed(const int16_t speed)
{
  OCR3C = (WHEEL_STOP_POSITION - speed) > MIN_SPEED_VALUE ? MIN_SPEED_VALUE:(WHEEL_STOP_POSITION - speed);
}

//************CONFIGURE PORTS************
//configure ports for input or output - specific to ATmega2560
void configure_ports_for_axon(void)
{
  //examples: configure ports for input or output
  //DDRB = 0x1F;  //configure B ports 0->4 for output (google search '0b00011111 to hex')
  //DDRC = 0x7F;  //configure all C ports for output, except 7 (0b01111111 to hex)
  //DDRD = 0xFE;  //configure all D ports for output, except 0 and 1 (RX/TX) FC
  //DDRC = _BV (PC0); //PC0 is digital output
  //0b76543210 <- port # order, 1 for output, 0 for input
  //to convert from binary to hex, google search '0b00011111 to hex'
  //DDRG &= ~_BV(PG5); //make port G5 an input pin
  //cbi(DDRG, PG5);  //set G5 as input pin

  //ANALOG PORTS
  //useful for sensors, SharpIR, photoresistors, etc.
  //analog ports can be configured as digital ports if desired
  DDRF = 0b00000000;  //configure all F ports for input				0x00
  PORTF = 0b00000000; //make sure pull-up resistors are turned off	0x00
  DDRK = 0b00000000;  //configure all K ports for input				0x00
  PORTK = 0b00000000; //make sure pull-up resistors are turned off	0x00

  //DIGITAL PORTS
  //useful for servos, PWM, LED's, UART, interrupts, timers
  DDRA = 0b11111111;  //configure ports for output
  //       ||||||||
  //       |||||||\___0: 
  //       ||||||\____1: 
  //       |||||\_____2: 
  //       ||||\______3: 
  //       |||\_______4: 
  //       ||\________5: 
  //       |\_________6: 
  //       \__________7: 
  //PORTB reserved for programmer (use programmer pins if you know what you are doing)
  DDRB = _BV (PB6);	//PB6 is LED, hold low to turn it on
  DDRC = 0b11111111;  //configure ports for output
  //       ||||||||
  //       |||||||\___0: 
  //       ||||||\____1: 
  //       |||||\_____2: 
  //       ||||\______3: 
  //       |||\_______4: 
  //       ||\________5: 
  //       |\_________6: 
  //       \__________7: 
  DDRD = 0b11111011;  //configure ports for output
  //       ||||||||
  //       |||||||\___0: 
  //       ||||||\____1: 
  //       |||||\_____2: serial RXD1, input  -> 0
  //       ||||\______3: serial TXD1, output -> 1
  //       |||\_______4: 
  //       ||\________5: 
  //       |\_________6: 
  //       \__________7: timer0
  DDRE = 0b11111110;  //configure ports for output
  //       ||||||||
  //       |||||||\___0: serial RXD0, input  -> 0
  //       ||||||\____1: serial TXD0, output -> 1
  //       |||||\_____2: 
  //       ||||\______3: 
  //       |||\_______4: 
  //       ||\________5: 
  //       |\_________6: 
  //       \__________7: 
  sbi(PORTG, PG5);  // enable pull-up resistor for v1f and later
  cbi(DDRG, PG5);	//PG5 is for the button, make a digital input
  //note that button actions are inverted between both versions!!!
  DDRH = 0b11111110;  //configure ports for output
  //       ||||||||
  //       |||||||\___0: serial RXD2, input  -> 0
  //       ||||||\____1: serial TXD2, output -> 1
  //       |||||\_____2: 
  //       ||||\______3: 
  //       |||\_______4: 
  //       ||\________5: 
  //       |\_________6: 
  //       \__________7: 
  DDRJ = 0b11111110;  //configure ports for output
  //       ||||||||
  //       |||||||\___0: serial RXD3, input  -> 0
  //       ||||||\____1: serial TXD3, output -> 1
  //       |||||\_____2: 
  //       ||||\______3: 
  //       |||\_______4: 
  //       ||\________5: 
  //       |\_________6: 
  //       \__________7: 
  //PORTL has no headers
};
//***************************************

void sleep(uint16_t t) {
  int i,j;
  for (i=0; i<t; ++i) {
    for (j=0; j<t; ++j) {
    }
  }
}

void LED_off_for_axon(void) {PORT_ON(PORTB,6);}
void LED_on_for_axon(void) {PORT_OFF(PORTB,6);}

void delay_cycles(unsigned long int cycles)
{
  cycles=cycles;//doubled frequency but too lazy to change times
  while(cycles > 0)
    cycles--;
}
int button_pressed(void)
{
  return (bit_is_clear(PING, 5));
}

void init_uart_for_axon() {
  uartInit();  // initialize the UART (serial port)
//  uartSetBaudRate(0, 38400); // set UARTE speed, for Bluetooth
  uartSetBaudRate(1, 115200); // set UARTD speed, for USB connection
//  uartSetBaudRate(2, 38400); // set UARTH speed
//  uartSetBaudRate(3, 38400); // set UARTJ speed, for Blackfin
}

void init_pwm3_fast_pwm_for_axon(const uint16_t prescaler, const uint16_t freq_in_ms) {
  TCCR3A = 0; // diable all PWM on Timer3 while we set it up

  // set TOP so that freq is every 20ms or 50hz
  ICR3 = get_top_fast_pwm(prescaler, (uint16_t)(1000/freq_in_ms)); 

  // configure timer 3 for fast pwm mode using ICR3, with prescale of 64
  // enable pwm on non-inverting mode
  TCCR3B = _BV(WGM33) | _BV(WGM32) | _BV(CS31) | _BV(CS30);
  // enable pin e3 & e5 for pwm
  TCCR3A = _BV(WGM31) | _BV(COM3A1) | _BV(COM3C1);

}

int main() {
  init_uart_for_axon();

  /* initialize rprintf system and configure uart1 (USB) for rprintf
   * uart must be initialized first */
  rprintfInit(uart1SendByte);

  // initialize the timer system (comment out ones you don't want)
//  timer0Init();
//  timer1Init();
//  timer2Init();
  timer3Init();
//  timer4Init();
//  timer5Init();
	
  configure_ports_for_axon(); // configure which ports are analog, digital, etc.

  a2dInit(); // initialize analog to digital converter (ADC)
  a2dSetPrescaler(ADC_PRESCALE_DIV32); // configure ADC scaling
  a2dSetReference(ADC_REFERENCE_AVCC); // configure ADC reference voltage

  LED_on_for_axon();

  rprintf("\r\nSystem Warming Up");

  //let system stabelize for X time
  for(int i=0;i<=16;i++)
  {
    delay_cycles(10000);
    rprintf(".");
  }

  LED_off_for_axon();
  /**************************************************/

  rprintf("\r\nSystem Ready, press button to start\r\n");

  while(!button_pressed()) {
    delay_cycles(5000);
  }

  init_pwm3_fast_pwm_for_axon(PWM3_PRESCALER, HS311_FREQ_MS);

  for(;;){
    // center servo for initialization
    set_left_wheel_speed(0);
    set_right_wheel_speed(0);
    LED_on_for_axon();
    sleep(800);
    LED_off_for_axon();
    set_left_wheel_speed(187);
    set_right_wheel_speed(187);
    sleep(1500);
    set_left_wheel_speed(0);
    set_right_wheel_speed(0);
    LED_on_for_axon();
    sleep(800);
    LED_off_for_axon();
    set_left_wheel_speed(187);
    set_right_wheel_speed(-187);
    sleep(1200);
  }
  return 0;
}
